/************************************************************************
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


This is demonstration code for controlling the outputs of a 
Chromation Systems 48 Channel Mono / 16 Channel RGB LED Controller 
When plugged into a computer USB it views it as an Emulated Serial Port(COM Port) and any software 
that can connect to serial devices, such as hyperterminal, can interact and change the LED outputs.

This is really basic and there is no data framing, so if a byte is missed or to many sent the outputs will be out of alignment

Start the Device Up -> Open Serial Program -> Connect to the COM port -> Send 48 bytes to
the device in order, if controlling RGB LEDs the outputs are in R-G-B order. First byte is used on output 1, 
second byte output 2... ect up to 48, then it starts again at 0. 8-bit numbers only.(0-255, standard)
 
RGB Example:
To turn LED 1 Red Send 255 -> 0 -> 0
To turn LED 2 Purple  send 255 -> 0 -> 255 after sending the data for LED 1
continue sending 3 bytes per LED

It is recommended that you first try ChromationSystems-c18-TLC5940-v2 project, and get familiar
with the TLC5940 library which is not covered here.

The free ColorMotion Compatible Demo Firmware can accept Live Data, same as this program but with
much better realiability and features. Please visit the website for details.

If you would like a full featured firmware and software for controlling RGB LEDs please visit:
http://www.chromationsystems.com/colormotion.html

Visit www.ChromationSystems.com/48channled.html for updates and more details

Released June 20, 2012 by www.ChromationSystems.com

//************************************************************************/


/** I N C L U D E S **********************************************************/

#include <p18f2550.h>

#include "HardwareProfile - ChromSys16RGBUSB.h"

#include "./USB/usb.h"
#include "./USB/usb_function_cdc.h"
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "usb_config.h"
#include "USB\usb_device.h"
#include "USB\usb.h"


typedef union tag8FLAGS { 
    unsigned char asByte;        // allows access to the full 8 bits 
    struct tag8BITS { 
  unsigned b0:1;    
  unsigned b1:1;    
  unsigned b2:1;    
  unsigned b3:1;
  unsigned b4:1;  
  unsigned b5:1;    
  unsigned b6:1;
  unsigned b7:1;
    }; 
} T8_FLAGS;


typedef union tag16FLAGS { 
    unsigned int asByte;        // allows access to the full 16 bits 
    struct tag16BITS { 
  unsigned b0:1;    
  unsigned b1:1;    
  unsigned b2:1;    
  unsigned b3:1;
  unsigned b4:1;  
  unsigned b5:1;    
  unsigned b6:1;
  unsigned b7:1;
  unsigned b8:1;    
  unsigned b9:1;    
  unsigned b10:1;    
  unsigned b11:1;
  unsigned b12:1;  
  unsigned b13:1;    
  unsigned b14:1;
  unsigned b15:1;    
    }; 
} T16_FLAGS; 


T8_FLAGS Flags;//define Flags union

/** V A R I A B L E S ********************************************************/
USB_HANDLE  lastTransmission;

char input_buffer[16]; 
unsigned char  NextUSBOut=0;
char USB_Out_Buffer[CDC_DATA_OUT_EP_SIZE];

unsigned char SendArray[8];

int i=0; //used in for()
int q=0;//used in for()

int bitCounter=0; //used in the TLC library
unsigned char byteCounter=0;//used in the TLC library
int GSCLK_Counter=0;//used in the TLC library
int DATA_Counter=0;//used in the TLC library

unsigned char LEDnum=0;
unsigned char DataCounter=0;	
unsigned char NewBuffer=0;		
unsigned char SwCount=0; //for switch debounce

		
//alter these Registers to change the colors.
unsigned char RedHoldVal[] ={255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255};
unsigned char GreenHoldVal[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
unsigned char BlueHoldVal[] = {255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

unsigned char TempRed=0;
unsigned char TempGreen=0;
unsigned char TempBlue=0;

//if you are short on RAM, these 2 can be combined with with minimal code altering
unsigned char DotCorrection[48]; //holds the dot correction data to be sent on startup, it is set in main()
unsigned char packedGreyScale[72];//holds the data to be sent after its been expanded from 8-bit to 12-bit number

/** P R I V A T E  P R O T O T Y P E S ***************************************/
static void InitializeSystem(void);
void ProcessIO(void);
void USBDeviceTasks(void);
void YourHighPriorityISRCode();
void YourLowPriorityISRCode();
void UserInit(void);
void set_timer(void);
void MainLoop(void);
unsigned char BitCheck(unsigned int var,unsigned int bitno);
void DataReceived(void);

void SwitchPress1(void);
void SwitchPress2(void);
void SwitchPress3(void);

unsigned char ReadEEPROM(unsigned char address);
void WriteEEPROM(unsigned int address, unsigned char data);

void InitTLC5940(unsigned char DCset);
void set_timer(void);
void XLATinterrupt(void);
void setInitGrayScales(void);
void PackData(void);


/** M A C R O ***************************************/

	#define bsf(var,bitno) var=(var | BtChk[bitno]);
	#define bcf(var,bitno) var=(var ^ BtChk[bitno]);

#define waitingForXLAT Flags.b1
#define updatePending Flags.b2
#define ToggleFlag Flags.b3
#define BounceFlag Flags.b4
#define PackTheData Flags.b5
#define CycleType Flags.b6
#define ChangeFlag Flags.b7

// ==== INTERUPTS=======

#if defined(__18CXX)
	#define PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
	#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
	#else	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
	#endif
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);        // See c018i.c in your C18 compiler dir
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
	#endif
	#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
	void Remapped_High_ISR (void)
	{
	     _asm goto YourHighPriorityISRCode _endasm
	}
	#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
	void Remapped_Low_ISR (void)
	{
	     _asm goto YourLowPriorityISRCode _endasm
	}
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	//Note: If this project is built while one of the bootloaders has
	#pragma code HIGH_INTERRUPT_VECTOR = 0x08
	void High_ISR (void)
	{
	     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#pragma code LOW_INTERRUPT_VECTOR = 0x18
	void Low_ISR (void)
	{
	     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#endif	//end of "#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER)"


//These are your actual interrupt handling routines.
	#pragma interrupt YourHighPriorityISRCode

	void YourHighPriorityISRCode()
	{
//run other code here
	}

#pragma interruptlow YourLowPriorityISRCode
void YourLowPriorityISRCode()
{
	if (INTCONbits.TMR0IF) //uses TMR0 as timing source
	{
	PackTheData=1;	
	XLATinterrupt(); //TLC required function
	INTCONbits.TMR0IF = 0;// Clear the timer0 interrupt flag
	INTCONbits.GIEL = 1;
	}
}//end function

	#pragma code

#include "ChromSysTLC5940lib.h"


void MainLoop(void)
{	
//This sample code Polls for new data, interrupts can also be setup in usb_config.h - Line 80
	if(USBDeviceState == CONFIGURED_STATE) //if USB is on and working
	{
	ProcessIO();//Required for USB, needs to run once and a while, a few times a sec or so.
	if(getsUSBUSART(input_buffer, 1)) DataReceived();	
	}

        #if defined(USB_POLLING) //device is polling
	        USBDeviceTasks();
        #endif	    
	    
/*	    
//to send constant strings(ROM) use
if(mUSBUSARTIsTxTrfReady()) putrsUSBUSART("Your Text Here"); 

//to send RAM variables use this, and from what I know it has to be this way
	SendArray[0]=12; //ROM or RAM
	SendArray[1]=DataCounter; //RAM
	SendArray[2]= LEDnum; //RAM
	SendArray[3]= GreenHoldVal[0]; //RAM
	if(USBUSARTIsTxTrfReady()) putUSBUSART(SendArray, 4); //sends 4 bytes that were just set, can be 1 to arraylength
*/    

if(Switch1==0) SwitchPress1();
if(Switch2==0) SwitchPress2();
if(Switch3==0) SwitchPress3();
	    
_asm
goto MainLoop
_endasm	    
}

void DataReceived(void)
{
	NewBuffer=input_buffer[0]; //move new byte into buffer

	switch(DataCounter)
	{
		case 0:
			RedHoldVal[LEDnum]= NewBuffer;
			break;
		case 1:
			GreenHoldVal[LEDnum]= NewBuffer;
			break;
		case 2:
			BlueHoldVal[LEDnum]= NewBuffer;
			break;	
	}
	DataCounter++;
	if(DataCounter==3)
	{
	if(LEDnum < 15) LEDnum++;
	else LEDnum=0;
	
	
	PackData();
	DataCounter=0;			
}
						
}//end function




void main(void)
{
	//Program starts here after initial setups
	
	    InitializeSystem();
	    
	#if defined(USB_POLLING)
        USBDeviceAttach();
   	 #endif

PackData();

_asm
goto MainLoop
_endasm	    
}	    

void SwitchPress1(void)
{
	if(Switch1 == 0) 
	{
	SwCount++;
	
	if(SwCount >=100)
	{
	SwCount=0;
	
	//what to do here?	
	//example turns all the LEDs green, in additon to their current color.
	for(i=0; i != 16; i++) GreenHoldVal[i]=255;
	PackData();
	
	while(Switch1==0) Nop();
	}
	else return;
	}
}//end Function


//===============================

void SwitchPress2(void)
{
	if(Switch2 == 0) 
	{
	SwCount++;
	
	if(SwCount >=100)
	{
	SwCount=0;
	
	//what to do here?
	
	//Turns all LED colors to 0/off	
	for(i=0; i != 16; i++) 
	{
	RedHoldVal[i]=0;
	GreenHoldVal[i]=0;	
	BlueHoldVal[i]=0;	
	}
	PackData();	
	
	while(Switch2==0) Nop();
	}
	else return;
	}
}//end Function

//===============================

void SwitchPress3(void)
{
	if(Switch3 == 0) 
	{
	SwCount++;
	
	if(SwCount >=100)
	{
	SwCount=0;
	
	//what to do here?
	
	//example turns all the LEDs blue, in additon to their current color.	
	for(i=0; i != 16; i++) BlueHoldVal[i]=255;
	PackData();
	
		
	while(Switch3==0) Nop();
	}
	else return;
	}
}//end Function



/******************************************************************************
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 *****************************************************************************/
void UserInit(void)
	{
	Flags.asByte=0x00; //clear all flags

	ADCON0 = 0x00;//disable unwanted hardware
	ADCON1 = 0x0F; 
	ADCON2 = 0x00;
	CMCON=0x07;

	TRISA = 0x00;
	TRISB = 0x00;
	TRISC = 0x00;


	TRISBbits.RB4=1; //setup switches
	TRISBbits.RB3=1;
	TRISBbits.RB2=1;

	ToggleFlag=0;
	BounceFlag=0;
	waitingForXLAT = 1; //flag used with TLC5940 Library
	updatePending=1; //flag is used if new data is pending


	PORTA=0x00;
	PORTB=0x00;
	PORTC=0x00;
	LATA = 0x00;
	LATB = 0x00;
	LATC = 0x00;

	PIR1bits.CCP1IF=0; //clear CCP int
	PIR2bits.CCP2IF=0; //clear CCP int

	INTCONbits.PEIE = 1; // Enable Peripheral interrupt 
	INTCONbits.GIE = 1;	// Enble Global interrupt, REQUIRED FOR LOW PRIORITY


//Check out the ChromationSystems-c18-TLC5940-v2 project for more information on
// Dot Correction and other important comments
	InitTLC5940(1); //Dot correction set to maximum
}//end UserInit





//==================================
// EEPROM read routine

unsigned char ReadEEPROM(unsigned char Address)
{
    EECON1=0;                   //ensure CFGS=0 and EEPGD=0 
    EEADR = Address;
    EECON1bits.RD = 1;
		return EEDATA; // Return with the data
}

//==================================
// EEPROM write routine

void WriteEEPROM(unsigned int Address, unsigned char Data)
{
static unsigned char GIE_Status; // Variable to save Global Interrupt Enable bit

//EEADRH = (unsigned char)(Address>>8); // Load the high byte of the EEPROM address
EEADR = (unsigned char)Address; // Load the low byte of the EEPROM address
EEDATA = Data; // Load the EEPROM data
EECON1bits.WREN = 1; // Enable EEPROM writes
GIE_Status = INTCONbits.GIE; // Save the Global Interrupt Enable bit
INTCONbits.GIE = 0; // Disable global interrupts
EECON2 = 0x55; // Required sequence to start the write cycle
EECON2 = 0xAA; // Required sequence to start the write cycle
EECON1bits.WR = 1; // Required sequence to start the write cycle
INTCONbits.GIE = GIE_Status; // Restore the Global Interrupt Enable bit
EECON1bits.WREN = 0; // Disable EEPROM writes
while (EECON1bits.WR); // Wait for the write cycle to complete
}


/********************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.                  
 *
 * Note:            None
 *******************************************************************/
static void InitializeSystem(void)
{
//	The USB specifications require that USB peripheral devices must never source
//	current onto the Vbus pin.  Additionally, USB peripherals should not source
//	current on D+ or D- when the host/hub is not actively powering the Vbus line.
//	When designing a self powered (as opposed to bus powered) USB peripheral
//	device, the firmware should make sure not to turn on the USB module and D+
//	or D- pull up resistor unless Vbus is actively powered.  Therefore, the
//	firmware needs some means to detect when Vbus is being powered by the host.
//	A 5V tolerant I/O pin can be connected to Vbus (through a resistor), and
// 	can be used to detect when Vbus is high (host actively powering), or low
//	(host is shut down or otherwise not supplying power).  The USB firmware
// 	can then periodically poll this I/O pin to know when it is okay to turn on
//	the USB module/D+/D- pull up resistor.  When designing a purely bus powered
//	peripheral device, it is not possible to source current on D+ or D- when the
//	host is not actively providing power on Vbus. Therefore, implementing this
//	bus sense feature is optional.  This firmware can be made to use this bus
//	sense feature by making sure "USE_USB_BUS_SENSE_IO" has been defined in the
//	HardwareProfile.h file.    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
//	If the host PC sends a GetStatus (device) request, the firmware must respond
//	and let the host know if the USB peripheral device is currently bus powered
//	or self powered.  See chapter 9 in the official USB specifications for details
//	regarding this request.  If the peripheral device is capable of being both
//	self and bus powered, it should not return a hard coded value for this request.
//	Instead, firmware should check if it is currently self or bus powered, and
//	respond accordingly.  If the hardware has been configured like demonstrated
//	on the PICDEM FS USB Demo Board, an I/O pin can be polled to determine the
//	currently selected power source.  On the PICDEM FS USB Demo Board, "RA2" 
//	is used for	this purpose.  If using this feature, make sure "USE_SELF_POWER_SENSE_IO"
//	has been defined in HardwareProfile.h, and that an appropriate I/O pin has been mapped
//	to it in HardwareProfile.h.
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif
    
    UserInit();

    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
    					//variables to known states.
}//end InitializeSystem



/******************************************************************************
 * Function:        void mySetLineCodingHandler(void)
 *
 * PreCondition:    USB_CDC_SET_LINE_CODING_HANDLER is defined
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function gets called when a SetLineCoding command
 *                  is sent on the bus.  This function will evaluate the request
 *                  and determine if the application should update the baudrate
 *                  or not.
 *
 * Note:            
 *
 *****************************************************************************/
#if defined(USB_CDC_SET_LINE_CODING_HANDLER)
void mySetLineCodingHandler(void)
{
    //If the request is not in a valid range
    if(cdc_notice.GetLineCoding.dwDTERate.Val > 115200)
    {
        //NOTE: There are two ways that an unsupported baud rate could be
        //handled.  The first is just to ignore the request and don't change
        //the values.  That is what is currently implemented in this function.
        //The second possible method is to stall the STATUS stage of the request.
        //STALLing the STATUS stage will cause an exception to be thrown in the 
        //requesting application.  Some programs, like HyperTerminal, handle the
        //exception properly and give a pop-up box indicating that the request
        //settings are not valid.  Any application that does not handle the
        //exception correctly will likely crash when this requiest fails.  For
        //the sake of example the code required to STALL the status stage of the
        //request is provided below.  It has been left out so that this demo
        //does not cause applications without the required exception handling
        //to crash.
        //---------------------------------------
        //USBStallEndpoint(0,1);
    }
    else
    {
        DWORD_VAL dwBaud;

        //Update the baudrate info in the CDC driver
        CDCSetBaudRate(cdc_notice.GetLineCoding.dwDTERate.Val);
        
        //Update the baudrate of the UART
            dwBaud.Val = (GetSystemClock()/4)/line_coding.dwDTERate.Val-1;
            SPBRG = dwBaud.v[0];
            SPBRGH = dwBaud.v[1];
    }
}
#endif



/********************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *
 * Note:            None
 *******************************************************************/
void ProcessIO(void)
{   
    //Blink the LEDs according to the USB device status

  //  BlinkUSBStatus(); //DEBUG


    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

	if((USBUSARTIsTxTrfReady()) && (NextUSBOut > 0))
	{
		putUSBUSART(&USB_Out_Buffer[0], NextUSBOut);
		NextUSBOut = 0;
	}

    CDCTxService();
}		//end ProcessIO



// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Call back that is invoked when a USB suspend is detected
 *
 * Note:            None
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
	//things to not work as intended.	
}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 *
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
    #if !defined(self_powered)
        if(U1OTGIRbits.ACTVIF)
        {
            IEC5bits.USB1IE = 0;
            U1OTGIEbits.ACTVIE = 0;
            IFS5bits.USB1IF = 0;
        
            //USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
            USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
            //USBSuspendControl = 0;
        }
    #endif
}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup from USB suspend 
 *					is detected.
 *
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be 
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 *
 * Note:            None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
    USBCheckCDCRequest();
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 *
 * Note:            None
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
    // Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 *
 * Note:            None
 *******************************************************************/
void USBCBInitEP(void)
{
    CDCInitEP();
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *					
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes 
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function should only be called when:
 *					
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET 
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.   
 *
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            Interrupt vs. Polling
 *                  -Primary clock
 *                  -Secondary clock ***** MAKE NOTES ABOUT THIS *******
 *                   > Can switch to primary first by calling USBCBWakeFromSuspend()
 
 *                  The modifiable section in this routine should be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of 1-13 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at lest 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do
    {
        delay_count--;
    }while(delay_count);
    USBResumeControl = 0;
}


/*******************************************************************
 * Function:        void USBCBEP0DataReceived(void)
 *
 * PreCondition:    ENABLE_EP0_DATA_RECEIVED_CALLBACK must be
 *                  defined already (in usb_config.h)
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called whenever a EP0 data
 *                  packet is received.  This gives the user (and
 *                  thus the various class examples a way to get
 *                  data that is received via the control endpoint.
 *                  This function needs to be used in conjunction
 *                  with the USBCBCheckOtherReq() function since 
 *                  the USBCBCheckOtherReq() function is the apps
 *                  method for getting the initial control transfer
 *                  before the data arrives.
 *
 * Note:            None
 *******************************************************************/
#if defined(ENABLE_EP0_DATA_RECEIVED_CALLBACK)
void USBCBEP0DataReceived(void)
{
}
#endif

/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * PreCondition:    None
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}

/** EOF main.c *************************************************/
